@@ -49,8 +49,14 @@ const char* messages[] = {
     "to be exported from a module must be defined with 'var'. The property "
     "access will work as previously for the time being, but please fix your "
     "code anyway.",
+
+    // PlatformSpecificTypelib:
+    ("{} has been moved to a separate platform-specific library. Please update "
+     "your code to use {} instead."),
 };
 
+static_assert(G_N_ELEMENTS(messages) == GjsDeprecationMessageId::LastValue);
+
 struct DeprecationEntry {
     GjsDeprecationMessageId id;
     std::string loc;
@@ -111,9 +117,9 @@ void _gjs_warn_deprecated_once_per_callsite(JSContext* cx,
     warn_deprecated_unsafe_internal(cx, id, messages[id]);
 }
 
-void _gjs_warn_deprecated_once_per_callsite(JSContext* cx,
-                                            GjsDeprecationMessageId id,
-                                            std::vector<const char*> args) {
+void _gjs_warn_deprecated_once_per_callsite(
+    JSContext* cx, GjsDeprecationMessageId id,
+    const std::vector<const char*>& args) {
     // In C++20, use std::format() for this
     std::string_view format_string{messages[id]};
     std::stringstream message;
@@ -126,7 +132,7 @@ void _gjs_warn_deprecated_once_per_callsite(JSContext* cx,
     while ((pos = format_string.find("{}", pos)) != std::string::npos) {
         if (args_ptr >= nargs_given) {
             g_critical("Only %zu format args passed for message ID %u",
-                       nargs_given, id);
+                       nargs_given, unsigned{id});
             return;
         }
 
@@ -136,11 +142,12 @@ void _gjs_warn_deprecated_once_per_callsite(JSContext* cx,
     }
     if (args_ptr != nargs_given) {
         g_critical("Excess %zu format args passed for message ID %u",
-                   nargs_given, id);
+                   nargs_given, unsigned{id});
         return;
     }
 
     message << format_string.substr(copied, std::string::npos);
 
-    warn_deprecated_unsafe_internal(cx, id, message.str().c_str());
+    std::string message_formatted = message.str();
+    warn_deprecated_unsafe_internal(cx, id, message_formatted.c_str());
 }
