@@ -116,7 +116,7 @@ bool gjs_string_to_utf8_n(JSContext* cx, JS::HandleString str, JS::UniqueChars*
         return false;
 
     size_t length = JS::GetDeflatedUTF8StringLength(linear);
-    char* bytes = js_pod_arena_malloc<char>(js::StringBufferArena, length + 1);
+    char* bytes = js_pod_malloc<char>(length + 1);
     if (!bytes)
         return false;
 
@@ -209,7 +209,7 @@ gjs_string_to_filename(JSContext      *context,
                        const JS::Value filename_val,
                        GjsAutoChar    *filename_string)
 {
-    GError *error;
+    GjsAutoError error;
 
     /* gjs_string_to_filename verifies that filename_val is a string */
 
@@ -233,7 +233,7 @@ gjs_string_from_filename(JSContext             *context,
                          JS::MutableHandleValue value_p)
 {
     gsize written;
-    GError *error;
+    GjsAutoError error;
 
     error = NULL;
     GjsAutoChar utf8_string = g_filename_to_utf8(filename_string, n_bytes,
@@ -241,9 +241,7 @@ gjs_string_from_filename(JSContext             *context,
     if (error) {
         gjs_throw(context,
                   "Could not convert UTF-8 string '%s' to a filename: '%s'",
-                  filename_string,
-                  error->message);
-        g_error_free(error);
+                  filename_string, error->message);
         return false;
     }
 
@@ -337,7 +335,7 @@ gjs_string_to_ucs4(JSContext       *cx,
         return true;
 
     size_t len;
-    GError *error = NULL;
+    GjsAutoError error;
 
     if (JS::StringHasLatin1Chars(str))
         return from_latin1(cx, str, ucs4_string_p, len_p);
@@ -361,7 +359,6 @@ gjs_string_to_ucs4(JSContext       *cx,
         if (*ucs4_string_p == NULL) {
             gjs_throw(cx, "Failed to convert UTF-16 string to UCS-4: %s",
                       error->message);
-            g_clear_error(&error);
             return false;
         }
         if (len_p != NULL)
@@ -393,14 +390,13 @@ gjs_string_from_ucs4(JSContext             *cx,
     }
 
     long u16_string_length;
-    GError *error = NULL;
+    GjsAutoError error;
 
     gunichar2* u16_string = g_ucs4_to_utf16(ucs4_string, n_chars, nullptr,
                                             &u16_string_length, &error);
     if (!u16_string) {
         gjs_throw(cx, "Failed to convert UCS-4 string to UTF-16: %s",
                   error->message);
-        g_error_free(error);
         return false;
     }
 
@@ -594,7 +590,7 @@ gjs_debug_object(JSObject * const obj)
 
     if (js::IsFunctionObject(obj)) {
         JSFunction* fun = JS_GetObjectFunction(obj);
-        JSString* display_name = JS_GetFunctionDisplayId(fun);
+        JSString* display_name = JS_GetMaybePartialFunctionDisplayId(fun);
         if (display_name && JS_GetStringLength(display_name))
             out << "<function " << gjs_debug_string(display_name);
         else
@@ -626,6 +622,15 @@ gjs_debug_object(JSObject * const obj)
     return out.str();
 }
 
+std::string gjs_debug_callable(JSObject* callable) {
+    if (JSFunction* fn = JS_GetObjectFunction(callable)) {
+        if (JSString* display_id = JS_GetMaybePartialFunctionDisplayId(fn))
+            return {"function " + gjs_debug_string(display_id)};
+        return {"unnamed function"};
+    }
+    return {"callable object " + gjs_debug_object(callable)};
+}
+
 std::string
 gjs_debug_value(JS::Value v)
 {
