@@ -1,18 +1,15 @@
 // SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
 // SPDX-FileCopyrightText: 2020 Andy Holmes <andrew.g.r.holmes@gmail.com>
 
-'use strict';
-
-const GLib = imports.gi.GLib;
-const Gio = imports.gi.Gio;
-
+import GLib from 'gi://GLib';
+import Gio from 'gi://Gio';
 
 /*
  * An XML DBus Interface
  */
 const ifaceXml = `
 <node>
-  <interface name="org.gnome.gjs.Test">
+  <interface name="org.cinnamon.cjs.Test">
     <method name="SimpleMethod"/>
     <method name="ComplexMethod">
       <arg type="s" direction="in" name="input"/>
@@ -88,7 +85,7 @@ let serviceSignalId = 0;
 function onBusAcquired(connection, _name) {
     // At this point you have acquired a connection to the bus, and you should
     // export your interfaces now.
-    serviceObj.dbus.export(connection, '/org/gnome/gjs/Test');
+    serviceObj.dbus.export(connection, '/org/cinnamon/cjs/Test');
 }
 
 function onNameAcquired(_connection, _name) {
@@ -116,7 +113,7 @@ function onNameLost(_connection, _name) {
 
 let ownerId = Gio.bus_own_name(
     Gio.BusType.SESSION,
-    'org.gnome.gjs.Test',
+    'org.cinnamon.cjs.Test',
     Gio.BusNameOwnerFlags.NONE,
     onBusAcquired,
     onNameAcquired,
