@@ -19,6 +19,8 @@
 
 #include <js/CallArgs.h>
 #include <js/ComparisonOperators.h>
+#include <js/ErrorReport.h>  // for JSEXN_TYPEERR
+#include <js/GCVector.h>     // for MutableHandleIdVector
 #include <js/Id.h>
 #include <js/MemoryFunctions.h>
 #include <js/Object.h>
@@ -26,14 +28,13 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 #include <js/Value.h>
-#include <jsapi.h>       // for JS_GetPrototype
-#include <jspubtd.h>     // for JSProto_TypeError
+#include <jsapi.h>  // for JS_GetPrototype
 
 #include "gi/arg-inl.h"
 #include "gi/cwrapper.h"
 #include "cjs/atoms.h"
 #include "cjs/context-private.h"
-#include "cjs/jsapi-class.h"  // IWYU pragma: keep
+#include "cjs/jsapi-class.h"
 #include "cjs/jsapi-util.h"
 #include "cjs/macros.h"
 #include "cjs/profiler-private.h"
@@ -635,12 +636,12 @@ class GIWrapperBase : public CWrapperPointerOps<Base> {
 
         if (expected_info) {
             gjs_throw_custom(
-                cx, JSProto_TypeError, nullptr,
+                cx, JSEXN_TYPEERR, nullptr,
                 "Object is of type %s.%s - cannot convert to %s.%s", priv->ns(),
                 priv->name(), g_base_info_get_namespace(expected_info),
                 g_base_info_get_name(expected_info));
         } else {
-            gjs_throw_custom(cx, JSProto_TypeError, nullptr,
+            gjs_throw_custom(cx, JSEXN_TYPEERR, nullptr,
                              "Object is of type %s.%s - cannot convert to %s",
                              priv->ns(), priv->name(),
                              g_type_name(expected_gtype));
