@@ -1,12 +1,24 @@
-const ByteArray = imports.byteArray;
-const {Gio, CjsPrivate, GLib} = imports.gi;
+// SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
+// SPDX-FileCopyrightText: 2008 litl, LLC
+
+const {Gio, CjsTestTools, GLib} = imports.gi;
+
+// Adapter for compatibility with pre-GLib-2.80
+let GioUnix;
+if (imports.gi.versions.GioUnix === '2.0') {
+    GioUnix = imports.gi.GioUnix;
+} else {
+    GioUnix = {
+        InputStream: Gio.UnixInputStream,
+    };
+}
 
 /* The methods list with their signatures.
  *
  * *** NOTE: If you add stuff here, you need to update the Test class below.
  */
 var TestIface = `<node>
-<interface name="org.gnome.gjs.Test">
+<interface name="org.cinnamon.cjs.Test">
 <method name="nonJsonFrobateStuff">
     <arg type="i" direction="in"/>
     <arg type="s" direction="out"/>
@@ -94,6 +106,7 @@ var TestIface = `<node>
 <property name="PropReadOnly" type="d" access="read" />
 <property name="PropWriteOnly" type="s" access="write" />
 <property name="PropReadWrite" type="v" access="readwrite" />
+<property name="PropPrePacked" type="a{sv}" access="read" />
 </interface>
 </node>`;
 
@@ -109,7 +122,7 @@ class Test {
         this._propReadWrite = PROP_READ_WRITE_INITIAL_VALUE;
 
         this._impl = Gio.DBusExportedObject.wrapJSObject(TestIface, this);
-        this._impl.export(Gio.DBus.session, '/org/gnome/gjs/Test');
+        this._impl.export(Gio.DBus.session, '/org/cinnamon/cjs/Test');
     }
 
     frobateStuff() {
@@ -139,6 +152,10 @@ class Test {
         return `${a} ${b} ${c} ${d} ${e}`;
     }
 
+    emitPropertyChanged(name, value) {
+        this._impl.emit_property_changed(name, value);
+    }
+
     emitSignal() {
         this._impl.emit_signal('signalFoo', GLib.Variant.new('(s)', ['foobar']));
     }
@@ -215,13 +232,19 @@ class Test {
         this._propReadWrite = value.deepUnpack();
     }
 
+    get PropPrePacked() {
+        return new GLib.Variant('a{sv}', {
+            member: GLib.Variant.new_string('value'),
+        });
+    }
+
     structArray() {
         return [[128, 123456], [42, 654321]];
     }
 
     fdIn(fdIndex, fdList) {
         const fd = fdList.get(fdIndex);
-        const stream = new Gio.UnixInputStream({fd, closeFd: true});
+        const stream = new GioUnix.InputStream({fd, closeFd: true});
         const bytes = stream.read_bytes(4096, null);
         return bytes;
     }
@@ -229,7 +252,7 @@ class Test {
     // Same as fdIn(), but implemented asynchronously
     fdIn2Async([fdIndex], invocation, fdList) {
         const fd = fdList.get(fdIndex);
-        const stream = new Gio.UnixInputStream({fd, closeFd: true});
+        const stream = new GioUnix.InputStream({fd, closeFd: true});
         stream.read_bytes_async(4096, GLib.PRIORITY_DEFAULT, null, (obj, res) => {
             const bytes = obj.read_bytes_finish(res);
             invocation.return_value(new GLib.Variant('(ay)', [bytes]));
@@ -237,14 +260,14 @@ class Test {
     }
 
     fdOut(bytes) {
-        const fd = CjsPrivate.open_bytes(bytes);
+        const fd = CjsTestTools.open_bytes(bytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         return [0, fdList];
     }
 
     fdOut2Async([bytes], invocation) {
         GLib.idle_add(GLib.PRIORITY_DEFAULT, function () {
-            const fd = CjsPrivate.open_bytes(bytes);
+            const fd = CjsTestTools.open_bytes(bytes);
             const fdList = Gio.UnixFDList.new_from_array([fd]);
             invocation.return_value_with_unix_fd_list(new GLib.Variant('(h)', [0]),
                 fdList);
@@ -260,6 +283,7 @@ describe('Exported DBus object', function () {
     var test;
     var proxy;
     let loop;
+    const expectedBytes = new TextEncoder().encode('some bytes');
 
     function waitForServerProperty(property, value = undefined, timeout = 500) {
         let waitId = GLib.timeout_add(GLib.PRIORITY_DEFAULT, timeout, () => {
@@ -282,7 +306,7 @@ describe('Exported DBus object', function () {
         loop = new GLib.MainLoop(null, false);
 
         test = new Test();
-        ownNameID = Gio.DBus.session.own_name('org.gnome.gjs.Test',
+        ownNameID = Gio.DBus.session.own_name('org.cinnamon.cjs.Test',
             Gio.BusNameOwnerFlags.NONE,
             name => {
                 log(`Acquired name ${name}`);
@@ -292,8 +316,8 @@ describe('Exported DBus object', function () {
                 log(`Lost name ${name}`);
             });
         loop.run();
-        new ProxyClass(Gio.DBus.session, 'org.gnome.gjs.Test',
-            '/org/gnome/gjs/Test',
+        new ProxyClass(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test',
             (obj, error) => {
                 expect(error).toBeNull();
                 proxy = obj;
@@ -323,6 +347,19 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call a method with async/await', async function () {
+        const [{hello}] = await proxy.frobateStuffAsync({});
+        expect(hello.deepUnpack()).toEqual('world');
+    });
+
+    it('can initiate a proxy with promise and call a method with async/await', async function () {
+        const asyncProxy = await ProxyClass.newAsync(Gio.DBus.session,
+            'org.cinnamon.cjs.Test', '/org/cinnamon/cjs/Test');
+        expect(asyncProxy).toBeInstanceOf(Gio.DBusProxy);
+        const [{hello}] = await asyncProxy.frobateStuffAsync({});
+        expect(hello.deepUnpack()).toEqual('world');
+    });
+
     it('can call a remote method when not using makeProxyWrapper', function () {
         let info = Gio.DBusNodeInfo.new_for_xml(TestIface);
         let iface = info.interfaces[0];
@@ -330,8 +367,8 @@ describe('Exported DBus object', function () {
         Gio.DBusProxy.new_for_bus(Gio.BusType.SESSION,
             Gio.DBusProxyFlags.DO_NOT_AUTO_START,
             iface,
-            'org.gnome.gjs.Test',
-            '/org/gnome/gjs/Test',
+            'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test',
             iface.name,
             null,
             (o, res) => {
@@ -361,6 +398,13 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can handle an exception thrown by a method with async/await', async function () {
+        GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
+            'JS ERROR: Exception in method call: alwaysThrowException: *');
+
+        await expectAsync(proxy.alwaysThrowExceptionAsync({})).toBeRejected();
+    });
+
     it('can still destructure the return value when an exception is thrown', function () {
         GLib.test_expect_message('Cjs', GLib.LogLevelFlags.LEVEL_WARNING,
             'JS ERROR: Exception in method call: alwaysThrowException: *');
@@ -391,6 +435,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('throws an exception when trying to call an async method that does not exist', async function () {
+        delete Test.prototype.thisDoesNotExist;
+        await expectAsync(proxy.thisDoesNotExistAsync()).toBeRejected();
+    });
+
     it('can pass a parameter to a remote method that is not a JSON object', function () {
         proxy.nonJsonFrobateStuffRemote(42, ([result], excp) => {
             expect(result).toEqual('42 it is!');
@@ -400,6 +449,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can pass a parameter to a method with async/await that is not a JSON object', async function () {
+        const [result] = await proxy.nonJsonFrobateStuffAsync(1);
+        expect(result).toEqual('Oops');
+    });
+
     it('can call a remote method with no in parameter', function () {
         proxy.noInParameterRemote(([result], excp) => {
             expect(result).toEqual('Yes!');
@@ -409,6 +463,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method with no in parameter', async function () {
+        const [result] = await proxy.noInParameterAsync();
+        expect(result).toEqual('Yes!');
+    });
+
     it('can call a remote method with multiple in parameters', function () {
         proxy.multipleInArgsRemote(1, 2, 3, 4, 5, ([result], excp) => {
             expect(result).toEqual('1 2 3 4 5');
@@ -418,6 +477,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method with multiple in parameters', async function () {
+        const [result] = await proxy.multipleInArgsAsync(1, 2, 3, 4, 5);
+        expect(result).toEqual('1 2 3 4 5');
+    });
+
     it('can call a remote method with no return value', function () {
         proxy.noReturnValueRemote(([result], excp) => {
             expect(result).not.toBeDefined();
@@ -427,6 +491,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method with no return value', async function () {
+        const [result] = await proxy.noReturnValueAsync();
+        expect(result).not.toBeDefined();
+    });
+
     it('can emit a DBus signal', function () {
         let handler = jasmine.createSpy('signalFoo');
         let id = proxy.connectSignal('signalFoo', handler);
@@ -443,6 +512,18 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can emit a DBus signal with async/await', async function () {
+        const handler = jasmine.createSpy('signalFoo');
+        const id = proxy.connectSignal('signalFoo', handler);
+        handler.and.callFake(() => proxy.disconnectSignal(id));
+
+        const [result] = await proxy.emitSignalAsync();
+        expect(result).not.toBeDefined();
+        expect(handler).toHaveBeenCalledTimes(1);
+        expect(handler).toHaveBeenCalledWith(jasmine.anything(),
+            jasmine.anything(), ['foobar']);
+    });
+
     it('can call a remote method with multiple return values', function () {
         proxy.multipleOutValuesRemote(function (result, excp) {
             expect(result).toEqual(['Hello', 'World', '!']);
@@ -452,6 +533,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method with multiple return values', async function () {
+        const results = await proxy.multipleOutValuesAsync();
+        expect(results).toEqual(['Hello', 'World', '!']);
+    });
+
     it('does not coalesce one array into the array of return values', function () {
         proxy.oneArrayOutRemote(([result], excp) => {
             expect(result).toEqual(['Hello', 'World', '!']);
@@ -461,6 +547,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('does not coalesce one array into the array of return values with async/await', async function () {
+        const [result] = await proxy.oneArrayOutAsync();
+        expect(result).toEqual(['Hello', 'World', '!']);
+    });
+
     it('does not coalesce an array of arrays into the array of return values', function () {
         proxy.arrayOfArrayOutRemote(([[a1, a2]], excp) => {
             expect(a1).toEqual(['Hello', 'World']);
@@ -471,6 +562,12 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('does not coalesce an array of arrays into the array of return values with async/await', async function () {
+        const [[a1, a2]] = await proxy.arrayOfArrayOutAsync();
+        expect(a1).toEqual(['Hello', 'World']);
+        expect(a2).toEqual(['World', 'Hello']);
+    });
+
     it('can return multiple arrays from a remote method', function () {
         proxy.multipleArrayOutRemote(([a1, a2], excp) => {
             expect(a1).toEqual(['Hello', 'World']);
@@ -481,6 +578,12 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can return multiple arrays from an async/await method', async function () {
+        const [a1, a2] = await proxy.multipleArrayOutAsync();
+        expect(a1).toEqual(['Hello', 'World']);
+        expect(a2).toEqual(['World', 'Hello']);
+    });
+
     it('handles a bad signature by throwing an exception', function () {
         proxy.arrayOutBadSigRemote(function (result, excp) {
             expect(excp).not.toBeNull();
@@ -489,6 +592,10 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('handles a bad signature in async/await by rejecting the promise', async function () {
+        await expectAsync(proxy.arrayOutBadSigAsync()).toBeRejected();
+    });
+
     it('can call a remote method that is implemented asynchronously', function () {
         let someString = 'Hello world!';
         let someInt = 42;
@@ -502,6 +609,14 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method that is implemented asynchronously', async function () {
+        const someString = 'Hello world!';
+        const someInt = 42;
+
+        const results = await proxy.echoAsync(someString, someInt);
+        expect(results).toEqual([someString, someInt]);
+    });
+
     it('can send and receive bytes from a remote method', function () {
         let someBytes = [0, 63, 234];
         someBytes.forEach(b => {
@@ -515,6 +630,14 @@ describe('Exported DBus object', function () {
         });
     });
 
+    it('can send and receive bytes from an async/await method', async function () {
+        let someBytes = [0, 63, 234];
+        await Promise.allSettled(someBytes.map(async b => {
+            const [byte] = await proxy.byteEchoAsync(b);
+            expect(byte).toEqual(b);
+        }));
+    });
+
     it('can call a remote method that returns an array of structs', function () {
         proxy.structArrayRemote(([result], excp) => {
             expect(excp).toBeNull();
@@ -524,6 +647,11 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can call an async/await method that returns an array of structs', async function () {
+        const [result] = await proxy.structArrayAsync();
+        expect(result).toEqual([[128, 123456], [42, 654321]]);
+    });
+
     it('can send and receive dicts from a remote method', function () {
         let someDict = {
             aDouble: new GLib.Variant('d', 10),
@@ -551,9 +679,21 @@ describe('Exported DBus object', function () {
         loop.run();
     });
 
+    it('can send and receive dicts from an async/await method', async function () {
+        // See notes in test above
+        const [result] = await proxy.dictEchoAsync({
+            aDouble: new GLib.Variant('d', 10),
+            anInteger: new GLib.Variant('i', 10.5),
+            aDoubleBeforeAndAfter: new GLib.Variant('d', 10.5),
+        });
+        expect(result).not.toBeNull();
+        expect(result['anInteger'].deepUnpack()).toEqual(10);
+        expect(result['aDoubleBeforeAndAfter'].deepUnpack()).toEqual(10.5);
+        expect(result['aDouble'].deepUnpack()).toBe(10.0);
+    });
+
     it('can call a remote method with a Unix FD', function (done) {
-        const expectedBytes = ByteArray.fromString('some bytes');
-        const fd = CjsPrivate.open_bytes(expectedBytes);
+        const fd = CjsTestTools.open_bytes(expectedBytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         proxy.fdInRemote(0, fdList, ([bytes], exc, outFdList) => {
             expect(exc).toBeNull();
@@ -563,9 +703,16 @@ describe('Exported DBus object', function () {
         });
     });
 
+    it('can call an async/await method with a Unix FD', async function () {
+        const fd = CjsTestTools.open_bytes(expectedBytes);
+        const fdList = Gio.UnixFDList.new_from_array([fd]);
+        const [bytes, outFdList] = await proxy.fdInAsync(0, fdList);
+        expect(outFdList).not.toBeDefined();
+        expect(bytes).toEqual(expectedBytes);
+    });
+
     it('can call an asynchronously implemented remote method with a Unix FD', function (done) {
-        const expectedBytes = ByteArray.fromString('some bytes');
-        const fd = CjsPrivate.open_bytes(expectedBytes);
+        const fd = CjsTestTools.open_bytes(expectedBytes);
         const fdList = Gio.UnixFDList.new_from_array([fd]);
         proxy.fdIn2Remote(0, fdList, ([bytes], exc, outFdList) => {
             expect(exc).toBeNull();
@@ -575,14 +722,21 @@ describe('Exported DBus object', function () {
         });
     });
 
+    it('can call an asynchronously implemented async/await method with a Unix FD', async function () {
+        const fd = CjsTestTools.open_bytes(expectedBytes);
+        const fdList = Gio.UnixFDList.new_from_array([fd]);
+        const [bytes, outFdList] = await proxy.fdIn2Async(0, fdList);
+        expect(outFdList).not.toBeDefined();
+        expect(bytes).toEqual(expectedBytes);
+    });
+
     function readBytesFromFdSync(fd) {
-        const stream = new Gio.UnixInputStream({fd, closeFd: true});
+        const stream = new GioUnix.InputStream({fd, closeFd: true});
         const bytes = stream.read_bytes(4096, null);
-        return ByteArray.fromGBytes(bytes);
+        return bytes.toArray();
     }
 
     it('can call a remote method that returns a Unix FD', function (done) {
-        const expectedBytes = ByteArray.fromString('some bytes');
         proxy.fdOutRemote(expectedBytes, ([fdIndex], exc, outFdList) => {
             expect(exc).toBeNull();
             const bytes = readBytesFromFdSync(outFdList.get(fdIndex));
@@ -591,8 +745,13 @@ describe('Exported DBus object', function () {
         });
     });
 
+    it('can call an async/await method that returns a Unix FD', async function () {
+        const [fdIndex, outFdList] = await proxy.fdOutAsync(expectedBytes);
+        const bytes = readBytesFromFdSync(outFdList.get(fdIndex));
+        expect(bytes).toEqual(expectedBytes);
+    });
+
     it('can call an asynchronously implemented remote method that returns a Unix FD', function (done) {
-        const expectedBytes = ByteArray.fromString('some bytes');
         proxy.fdOut2Remote(expectedBytes, ([fdIndex], exc, outFdList) => {
             expect(exc).toBeNull();
             const bytes = readBytesFromFdSync(outFdList.get(fdIndex));
@@ -601,19 +760,35 @@ describe('Exported DBus object', function () {
         });
     });
 
+    it('can call an asynchronously implemented asyc/await method that returns a Unix FD', async function () {
+        const [fdIndex, outFdList] = await proxy.fdOut2Async(expectedBytes);
+        const bytes = readBytesFromFdSync(outFdList.get(fdIndex));
+        expect(bytes).toEqual(expectedBytes);
+    });
+
     it('throws an exception when not passing a Gio.UnixFDList to a method that requires one', function () {
         expect(() => proxy.fdInRemote(0, () => {})).toThrow();
     });
 
+    it('rejects the promise when not passing a Gio.UnixFDList to an async method that requires one', async function () {
+        await expectAsync(proxy.fdInAsync(0)).toBeRejected();
+    });
+
     it('throws an exception when passing a handle out of range of a Gio.UnixFDList', function () {
         const fdList = new Gio.UnixFDList();
         expect(() => proxy.fdInRemote(0, fdList, () => {})).toThrow();
     });
 
+    it('rejects the promise when async passing a handle out of range of a Gio.UnixFDList', async function () {
+        const fdList = new Gio.UnixFDList();
+        await expectAsync(proxy.fdInAsync(0, fdList)).toBeRejected();
+    });
+
     it('Has defined properties', function () {
-        expect(proxy.hasOwnProperty('PropReadWrite')).toBeTruthy();
-        expect(proxy.hasOwnProperty('PropReadOnly')).toBeTruthy();
-        expect(proxy.hasOwnProperty('PropWriteOnly')).toBeTruthy();
+        expect(Object.hasOwn(proxy, 'PropReadWrite')).toBeTruthy();
+        expect(Object.hasOwn(proxy, 'PropReadOnly')).toBeTruthy();
+        expect(Object.hasOwn(proxy, 'PropWriteOnly')).toBeTruthy();
+        expect(Object.hasOwn(proxy, 'PropPrePacked')).toBeTruthy();
     });
 
     it('reading readonly property works', function () {
@@ -658,4 +833,118 @@ describe('Exported DBus object', function () {
 
         expect(proxy.PropReadOnly).toBe(PROP_READ_ONLY_INITIAL_VALUE);
     });
+
+    it('Reading a property that prepacks the return value works', function () {
+        expect(proxy.PropPrePacked.member).toBeInstanceOf(GLib.Variant);
+        expect(proxy.PropPrePacked.member.get_type_string()).toEqual('s');
+    });
+
+    it('Marking a property as invalidated works', function () {
+        let changedProps = {};
+        let invalidatedProps = [];
+
+        proxy.connect('g-properties-changed', (proxy_, changed, invalidated) => {
+            changedProps = changed.deepUnpack();
+            invalidatedProps = invalidated;
+            loop.quit();
+        });
+
+        test.emitPropertyChanged('PropReadOnly', null);
+        loop.run();
+
+        expect(changedProps).not.toContain('PropReadOnly');
+        expect(invalidatedProps).toContain('PropReadOnly');
+    });
+});
+
+
+describe('DBus Proxy wrapper', function () {
+    let loop;
+    let wasPromise;
+    let writerFunc;
+
+    beforeAll(function () {
+        loop = new GLib.MainLoop(null, false);
+
+        wasPromise = Gio.DBusProxy.prototype._original_init_async instanceof Function;
+        Gio._promisify(Gio.DBusProxy.prototype, 'init_async');
+
+        writerFunc = jasmine.createSpy(
+            'log writer func', (level, fields) => {
+                const decoder = new TextDecoder('utf-8');
+                const domain = decoder.decode(fields?.GLIB_DOMAIN);
+                const message = `${decoder.decode(fields?.MESSAGE)}\n`;
+                level |= GLib.LogLevelFlags.FLAG_RECURSION;
+                GLib.log_default_handler(domain, level, message, null);
+                return GLib.LogWriterOutput.HANDLED;
+            });
+
+        writerFunc.and.callThrough();
+        GLib.log_set_writer_func(writerFunc);
+    });
+
+    beforeEach(function () {
+        writerFunc.calls.reset();
+    });
+
+    it('init failures are reported in sync mode', function () {
+        const cancellable = new Gio.Cancellable();
+        cancellable.cancel();
+        expect(() => new ProxyClass(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test',
+            Gio.DBusProxyFlags.NONE,
+            cancellable)).toThrow();
+    });
+
+    it('init failures are reported in async mode', function () {
+        const cancellable = new Gio.Cancellable();
+        cancellable.cancel();
+        const initDoneSpy = jasmine.createSpy(
+            'init finish func', () => loop.quit());
+        initDoneSpy.and.callThrough();
+        new ProxyClass(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test',
+            initDoneSpy, cancellable, Gio.DBusProxyFlags.NONE);
+        loop.run();
+
+        expect(initDoneSpy).toHaveBeenCalledTimes(1);
+        const {args: callArgs} = initDoneSpy.calls.mostRecent();
+        expect(callArgs.at(0)).toBeNull();
+        expect(callArgs.at(1).matches(
+            Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED)).toBeTrue();
+    });
+
+    it('can init a proxy asynchronously when promisified', function () {
+        new ProxyClass(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test',
+            () => loop.quit(),
+            Gio.DBusProxyFlags.NONE);
+        loop.run();
+
+        expect(writerFunc).not.toHaveBeenCalled();
+    });
+
+    it('can create a proxy from a promise', async function () {
+        const proxyPromise = ProxyClass.newAsync(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test');
+        await expectAsync(proxyPromise).toBeResolved();
+    });
+
+    it('can create fail a proxy from a promise', async function () {
+        const cancellable = new Gio.Cancellable();
+        cancellable.cancel();
+        const proxyPromise = ProxyClass.newAsync(Gio.DBus.session, 'org.cinnamon.cjs.Test',
+            '/org/cinnamon/cjs/Test', cancellable);
+        await expectAsync(proxyPromise).toBeRejected();
+    });
+
+    afterAll(function () {
+        if (!wasPromise) {
+            // Remove stuff added by Gio._promisify, this can be not needed in future
+            // nor should break other tests, but we didn't want to depend on those.
+            expect(Gio.DBusProxy.prototype._original_init_async).toBeInstanceOf(Function);
+            Gio.DBusProxy.prototype.init_async = Gio.DBusProxy.prototype._original_init_async;
+            delete Gio.DBusProxy.prototype._original_init_async;
+        }
+    });
 });
